@@ -9,11 +9,11 @@ File::Next - File-finding iterator
 
 =head1 VERSION
 
-Version 0.36
+Version 0.38
 
 =cut
 
-our $VERSION = '0.36';
+our $VERSION = '0.38';
 
 =head1 SYNOPSIS
 
@@ -30,8 +30,8 @@ It's lightweight and has no non-core prerequisites.
 
 =head1 OPERATIONAL THEORY
 
-Each of the public functions in File::Next returns an iterator that
-will walk through a directory tree.  The simplest use case is:
+The two major functions, I<files()> and I<dirs()>, return an iterator
+that will walk through a directory tree.  The simplest use case is:
 
     use File::Next;
 
@@ -60,7 +60,49 @@ be a hashref of parameters.
 
 Note that the iterator will only return files, not directories.
 
-=head1 PARAMETERS
+=head1 FUNCTIONS
+
+=head2 files( { \%parameters }, @starting_points )
+
+Returns an iterator that walks directories starting with the items
+in I<@starting_points>.  Each call to the iterator returns another file.
+
+=head2 dirs( { \%parameters }, @starting_points )
+
+Returns an iterator that walks directories starting with the items
+in I<@starting_points>.  Each call to the iterator returns another
+directory.
+
+=head2 sort_standard( $a, $b )
+
+A sort function for passing as a C<sort_files> parameter:
+
+    my $iter = File::Next::files( {
+        sort_files => \&File::Next::sort_standard,
+    }, 't/swamp' );
+
+This function is the default, so the code above is identical to:
+
+    my $iter = File::Next::files( {
+        sort_files => 1,
+    }, 't/swamp' );
+
+=head2 sort_reverse( $a, $b )
+
+Same as C<sort_standard>, but in reverse.
+
+=head2 reslash( $path )
+
+Takes a path with all forward slashes and rebuilds it with whatever
+is appropriate for the platform.  For example 'foo/bar/bat' will
+become 'foo\bar\bat' on Windows.
+
+This is really just a convenience function.  I'd make it private,
+but F<ack> wants it, too.
+
+=cut
+
+=head1 CONSTRUCTOR PARAMETERS
 
 =head2 file_filter -> \&file_filter
 
@@ -87,6 +129,8 @@ These are analogous to the same variables in L<File::Find>.
 
 By default, the I<file_filter> is C<sub {1}>, or "all files".
 
+This filter has no effect if your iterator is only returning directories.
+
 =head2 descend_filter => \&descend_filter
 
 The descend_filter lets you check to see if the iterator should
@@ -107,9 +151,9 @@ a collection of variables.
 =back
 
 The descend filter is NOT applied to any directory names specified
-in the constructor.  For example,
+in as I<@starting_points> in the constructor.  For example,
 
-    my $iter = File::Find::files( { descend_filter => sub{0} }, '/tmp' );
+    my $iter = File::Next::files( { descend_filter => sub{0} }, '/tmp' );
 
 always descends into I</tmp>, as you would expect.
 
@@ -131,33 +175,14 @@ Note that the parms passed in to the sub are arrayrefs, where $a->[0]
 is the directory name, $a->[1] is the file name and $a->[2] is the
 full path.  Typically you're going to be sorting on $a->[2].
 
-=head1 FUNCTIONS
+=head2 follow_symlinks => [ 0 | 1 ]
 
-=head2 files( { \%parameters }, @starting points )
+If set to false, the iterator will ignore any files and directories
+that are actually symlinks.  This has no effect on non-Unixy systems
+such as Windows.  By default, this is true.
 
-Returns an iterator that walks directories starting with the items
-in I<@starting_points>.
-
-All file-finding in this module is adapted from Mark Jason Dominus'
-marvelous I<Higher Order Perl>, page 126.
-
-=head2 sort_standard( $a, $b )
-
-A sort function for passing as a C<sort_files> parameter:
-
-    my $iter = File::Next::files( {
-        sort_files => \&File::Next::sort_reverse
-    }, 't/swamp' );
-
-This function is the default, so the code above is identical to:
-
-    my $iter = File::Next::files( {
-        sort_files => \&File::Next::sort_reverse
-    }, 't/swamp' );
-
-=head2 sort_reverse( $a, $b )
-
-Same as C<sort_standard>, but in reverse.
+Note that this filter does not apply to any of the I<@starting_points>
+passed in to the constructor.
 
 =cut
 
@@ -172,52 +197,21 @@ our %skip_dirs;
 
 BEGIN {
     %files_defaults = (
-        file_filter => undef,
-        descend_filter => undef,
-        error_handler => sub { CORE::die @_ },
-        sort_files => undef,
+        file_filter     => undef,
+        descend_filter  => undef,
+        error_handler   => sub { CORE::die @_ },
+        sort_files      => undef,
+        follow_symlinks => 1,
     );
     %skip_dirs = map {($_,1)} (File::Spec->curdir, File::Spec->updir);
 }
 
-=for internal
-
-The C<@queue> that gets passed around is an array that has three
-elements for each of the entries in the queue: $dir, $file and
-$fullpath.  Items must be pushed and popped off the queue three at
-a time (spliced, really).
-
-=cut
-
 sub files {
-    my $passed_parms = ref $_[0] eq 'HASH' ? {%{+shift}} : {}; # copy parm hash
-    my %passed_parms = %{$passed_parms};
-
-    my $parms = {};
-    for my $key ( keys %files_defaults ) {
-        $parms->{$key} = delete( $passed_parms{$key} ) || $files_defaults{$key};
-    }
-
-    # Any leftover keys are bogus
-    for my $badkey ( keys %passed_parms ) {
-        $parms->{error_handler}->( "Invalid parameter passed to files(): $badkey" );
-    }
-
-    my @queue;
-    for ( @_ ) {
-        my $start = reslash( $_ );
-        if (-d $start) {
-            push @queue, ($start,undef,$start);
-        }
-        else {
-            push @queue, (undef,$start,$start);
-        }
-    }
+    my ($parms,@queue) = _setup( \%files_defaults, @_ );
 
     return sub {
         while (@queue) {
             my ($dir,$file,$fullpath) = splice( @queue, 0, 3 );
-
             if (-f $fullpath) {
                 if ( $parms->{file_filter} ) {
                     local $_ = $file;
@@ -236,12 +230,98 @@ sub files {
     }; # iterator
 }
 
-=for private _candidate_files( $parms, $dir )
+sub dirs {
+    my ($parms,@queue) = _setup( \%files_defaults, @_ );
+
+    return sub {
+        while (@queue) {
+            my ($dir,$file,$fullpath) = splice( @queue, 0, 3 );
+            if (-d $fullpath) {
+                unshift( @queue, _candidate_files( $parms, $fullpath ) );
+                return $fullpath;
+            }
+        } # while
+
+        return;
+    }; # iterator
+}
+
+sub sort_standard($$)   { return $_[0]->[1] cmp $_[1]->[1] }; ## no critic (ProhibitSubroutinePrototypes)
+sub sort_reverse($$)    { return $_[1]->[1] cmp $_[0]->[1] }; ## no critic (ProhibitSubroutinePrototypes)
+
+sub reslash {
+    my $path = shift;
+
+    my @parts = split( /\//, $path );
+
+    return $path if @parts < 2;
+
+    return File::Spec->catfile( @parts );
+}
+
+
+=head1 PRIVATE FUNCTIONS
+
+=head2 _setup( $default_parms, @whatever_was_passed_to_files() )
+
+Handles all the scut-work for setting up the parms passed in.
+
+Returns a hashref of operational parameters, combined between
+I<$passed_parms> and I<$defaults>, plus the queue.
+
+The queue prep stuff takes the strings in I<@starting_points> and
+puts them in the format that queue needs.
+
+The C<@queue> that gets passed around is an array that has three
+elements for each of the entries in the queue: $dir, $file and
+$fullpath.  Items must be pushed and popped off the queue three at
+a time (spliced, really).
+
+=cut
+
+sub _setup {
+    my $defaults = shift;
+    my $passed_parms = ref $_[0] eq 'HASH' ? {%{+shift}} : {}; # copy parm hash
+
+    my %passed_parms = %{$passed_parms};
+
+    my $parms = {};
+    for my $key ( keys %$defaults ) {
+        $parms->{$key} =
+            exists $passed_parms{$key}
+                ? delete $passed_parms{$key}
+                : $defaults->{$key};
+    }
+
+    # Any leftover keys are bogus
+    for my $badkey ( keys %passed_parms ) {
+        my $sub = (caller(1))[3];
+        $parms->{error_handler}->( "Invalid parameter passed to $sub(): $badkey" );
+    }
+
+    # If it's not a code ref, assume standard sort
+    if ( $parms->{sort_files} && ( ref($parms->{sort_files}) ne 'CODE' ) ) {
+        $parms->{sort_files} = \&sort_standard;
+    }
+    my @queue;
+
+    for ( @_ ) {
+        my $start = reslash( $_ );
+        if (-d $start) {
+            push @queue, ($start,undef,$start);
+        }
+        else {
+            push @queue, (undef,$start,$start);
+        }
+    }
+
+    return ($parms,@queue);
+}
+
+=head2 _candidate_files( $parms, $dir )
 
 Pulls out the files/dirs that might be worth looking into in I<$dir>.
 If I<$dir> is the empty string, then search the current directory.
-This is different than explicitly passing in a ".", because that
-will get prepended to the path names.
 
 I<$parms> is the hashref of parms passed into File::Next constructor.
 
@@ -263,6 +343,10 @@ sub _candidate_files {
 
         # Only do directory checking if we have a descend_filter
         my $fullpath = File::Spec->catdir( $dir, $file );
+        if ( !$parms->{follow_symlinks} ) {
+            next if -l $fullpath;
+        }
+
         if ( $parms->{descend_filter} && -d $fullpath ) {
             local $File::Next::dir = $fullpath;
             local $_ = $file;
@@ -271,8 +355,6 @@ sub _candidate_files {
         push( @newfiles, $dir, $file, $fullpath );
     }
     if ( my $sub = $parms->{sort_files} ) {
-        $sub = \&sort_standard unless ref($sub) eq 'CODE';
-
         my @triplets;
         while ( @newfiles ) {
             push @triplets, [splice( @newfiles, 0, 3 )];
@@ -283,30 +365,6 @@ sub _candidate_files {
     return @newfiles;
 }
 
-sub sort_standard($$)   { return $_[0]->[1] cmp $_[1]->[1] }; ## no critic (ProhibitSubroutinePrototypes)
-sub sort_reverse($$)    { return $_[1]->[1] cmp $_[0]->[1] }; ## no critic (ProhibitSubroutinePrototypes)
-
-
-=head2 reslash( $path )
-
-Takes a path with all forward slashes and rebuilds it with whatever
-is appropriate for the platform.  For example 'foo/bar/bat' will
-become 'foo\bar\bat' on Windows.
-
-This is really just a convenience function.
-
-=cut
-
-sub reslash {
-    my $path = shift;
-
-    my @parts = split( /\//, $path );
-
-    return $path if @parts < 2;
-
-    return File::Spec->catfile( @parts );
-}
-
 =head1 AUTHOR
 
 Andy Lester, C<< <andy at petdance.com> >>
@@ -353,9 +411,12 @@ L<https://file-next.googlecode.com/svn/trunk>
 
 =head1 ACKNOWLEDGEMENTS
 
+All file-finding in this module is adapted from Mark Jason Dominus'
+marvelous I<Higher Order Perl>, page 126.
+
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2006 Andy Lester, all rights reserved.
+Copyright 2006-2007 Andy Lester, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
